@@ -23,7 +23,7 @@ try:
 except:
     print("Unable to import Ipyvolume")
 
-__VERSION__ = "0.0.22"
+__VERSION__ = "0.0.35"
 
 """
     A module to analyze cytosim simulations
@@ -41,60 +41,75 @@ __VERSION__ = "0.0.22"
 #   Analysis are passed through dictionaries of functions { 'object_name' : { 'property' :  function } }
 
 
-
+### This should definitely be in a yaml file
+# No really, how stupid is that !!!!
+__N_DIM__= 3
+__NAMED_SETS__ = {"fiber": Fiber_set, "couple": Couple_set}
 
 export_analysis=an.export_analysis
 
 # Each frame contains a fibers, couples, and stuff
 class Frame():
     """
-    A frame is a data structure containing several Fibers, Couples, Space, etc.
-        frame.fibers is a dictionary, e.g. {'microtubules' : list_MT] }
+    A frame is a data structure containing several objects: Fibers, Couples, Space, etc.
+
+        frame.objects is a dictionary index by types, of dictionaries indexed by names, of objecs.
+        e.g. frame.objects["fiber"]["microtubule"] = list_MT
         where list_MT = [MT1, MT2, ... ], with MT1, MT2, ... being instances of class Fiber
 
     """
-    def __init__(self, *args, number=None, **kwargs):
-        self.fibers = {}
-        self.couples = {}
-        #self.spaces = {}
+    def __init__(self, *args, number=None, blocks = None, **kwargs):
         self.objects = {}
-
-        self.analysis = None
+        self.analysis = {}
         self.id = number
-        self.initialize(*args, **kwargs)
 
-        self.all_sets = [self.fibers, self.couples, self.objects]
+        if blocks is not None:
+            self.initialize(*args, blocks=blocks, **kwargs)
+        else:
+            print("Empty frame")
 
     def initialize(self, *args,
-                   fibers_lines={}, couples_lines={}, objects_lines={},
-                   config=None, options={}, **kwargs):
-
-        # Fibers factory :
-        for name, lines_dict in fibers_lines.items():
-            opt = an.get_options(options, name)
-            self.fibers[name] = Fiber_set(name=name, reports=lines_dict, config=config, **opt)
-
-        # Couples factory
-        for name, lines_dict in couples_lines.items():
-            opt = an.get_options(options, name)
-            self.couples[name] = Couple_set(name=name, reports=lines_dict, config=config, **opt)
-
-        # Custom objects factory
-        for name, lines_dict in objects_lines.items():
-            opt = an.get_options(options, name)
-            self.objects[name] = Object_set(name=name, reports=lines_dict, config=config, **opt)
-
-        # and so on...
+                   blocks =None,
+                   config=None, options=None, dim=__N_DIM__, **kwargs):
+        """
+        Actually create the objects inside the frame
+        """
+        if options is None:
+            options = {}
+        if blocks is None:
+            blocks = {}
 
+        for key, block in blocks.items():
+            self.objects[key] = {}
+            opt = {"type": key}
+            for name, lines_dict in block.items():
+                opt.update(an.get_options(options, name=name, type=key))
+                try:
+                    self.objects[key][name] = opt["set"](name=name, dim=dim,
+                                                         reports=lines_dict, config=config, **opt)
+                except:
+                    if key in __NAMED_SETS__.keys():
+                        self.objects[key][name] = __NAMED_SETS__[key](name=name, dim=dim,
+                                                                      reports=lines_dict, config=config, **opt)
+                    else:
+                        self.objects[key][name] = Object_set(name=name, dim=dim,
+                                                             reports=lines_dict, config=config, **opt)
 
     def make_analysis(self, *args, **kwargs):
+        """
+            A wrapper for object analysis
+        """
+        for key, object in self.objects.items():
+            #analysis = an.make_analysis(object, *args, analyzer= analyzer, **kwargs)
+            analysis = an.make_analysis(object, *args, **kwargs)
+            for name, ana in analysis.items():
+                self.analysis[name] = ana
 
-        self.fibers_analysis = an.make_analysis(self.fibers,*args,**kwargs)
-        self.couples_analysis = an.make_analysis(self.couples,*args,**kwargs)
-        self.objects_analysis = an.make_analysis(self.objects, *args, **kwargs)
-
-    def plot(self,*args, options=None, **kwargs):
-        for set in self.all_sets:
+    def plot(self, *args, options=None, **kwargs):
+        """
+            A wrapper for object plotting
+        """
+        for set in self.objects.values():
             for name, set in set.items():
                 opts={}
                 if options is not None:
@@ -103,17 +118,22 @@ class Frame():
                     except:
                         opts = {}
                 set.plot(*args, **opts,  **kwargs)
-
         return None
 
     def hide(self):
-        for set in self.all_sets:
+        """
+            Hides a frame from display
+        """
+        for set in self.objects.values():
             for name, set in set.items():
                 for plot in set.plots:
                     plot.visible=False
 
     def show(self):
-        for set in self.all_sets:
+        """
+            Shows a frame in display
+        """
+        for set in self.objects.values():
             for name, set in set.items():
                 for plot in set.plots:
                     plot.visible = True
@@ -126,67 +146,85 @@ class Simulation(list):
         eg : config=config.cym, fibers_reports={ "microtubule" : { "points" : "fiber_points.txt" } }
         ...
     """
-    def __init__(self,*args, **kwargs):
+    def __init__(self, *args, dim = __N_DIM__, **kwargs):
         list.__init__(self)
         self.config=None
         self.analysis=None
         self.frames_analysis=None
         self.properties={}
         self.name='frame'
+        self.type = None
         self.id = 0
         self.n_frames=0
         self.shown = False
+        self.dim = dim
         # Actual initialization
         self.initialize(*args,**kwargs)
 
-    # Initialization step
-    def initialize(self, *args,
-                   fibers_report={}, couples_report={}, objects_report={},
-                   config=None, **kwargs):
 
+    # Initialization step
+    def initialize(self, *args, config=None, options=None, **kwargs):
         """
             We actually build the simulation here
             We will make the frames from the report files using iterators over these files
 
-            x_iter is a dictionary of dictiionaries of iterators :
+            iters is a dictionary of dictionary of dictionaries of iterators :
             e.g.
-            fibers_iter = { "microtubule" : { "points" : iterator_to_fiber_points } }
+            iters["fiber] = { "microtubule" : { "points" : iterator_to_fiber_points } }
         """
+        if options is None:
+            options = {}
+
+        # A dictionary of reports
+        reports = {}
+        # A dictionary of iterators for reports
+        iters = {}
 
         if config is not None:
             self.read_config(config, *args, **kwargs)
 
-        fibers_iter = an.make_iter_dict(fibers_report)
-        objects_iter = an.make_iter_dict(objects_report)
-        couples_iter = an.make_iter_dict(couples_report)
+        # looking at the reports we have
+        for key, item in kwargs.items():
+            if key.endswith("_report"):
+                name = key.split("_")[0]
+                if name.endswith("s"):
+                    typename = name[:-1]
+                else:
+                    typename = name
+                # Here is a small trick to incorporate in options things of the like type_options = {...}
+                tt = "%s_options"%name
+                if tt in kwargs.keys():
+                    try:
+                        options[typename].update(kwargs[tt])
+                        print(options)
+                    except:
+                        options[typename] = kwargs[tt]
+                reports[typename] = item
 
+        # Initializing the iters
+        for key in reports.keys():
+            iters[key] = an.make_iter_dict(reports[key])
+            an.make_block_dict(iters[key])
 
-
-        # We skip the first lines until "% frames"
-        # Treat this as a dirty hack if you want.
-        fibers_block, f_flag = an.make_block_dict(fibers_iter)
-        couples_block, c_flag = an.make_block_dict(couples_iter)
-        objects_block, o_flag = an.make_block_dict(objects_iter)
-
-        # Now we iterate upon the config files
         keep = True
+        flags = {}
         while keep:
+            blocks = {}
+            for key, iter in iters.items():
+                blocks[key], flags[key] = an.make_block_dict(iter)
 
-            fibers_block, f_flag = an.make_block_dict(fibers_iter)
-            couples_block, c_flag = an.make_block_dict(couples_iter)
-            objects_block, o_flag = an.make_block_dict(objects_iter)
-
-            if not f_flag and not c_flag and not o_flag:
+            if not any(flags.values()):
                 keep = False
             else:
-                self.append(Frame(*args, number=len(self),
-                                  fibers_lines=fibers_block, couples_lines=couples_block, objects_lines=objects_block,
-                                  config=self.config, **kwargs))
+                self.append(Frame(*args, number=len(self),blocks=blocks, options=options,
+                                  config=self.config, dim=self.dim, **kwargs))
 
         self.n_frames = len(self)
 
 
-    def read_config(self, config, simul_props={}, **kwargs ):
+    def read_config(self, config, simul_props=None, **kwargs ):
+        if simul_props is None:
+            simul_props = {}
 
         self.config = rc.parse(config)
         com = ['set', 'simul', '*']
@@ -195,7 +233,7 @@ class Simulation(list):
 
 
     def analyze(self, frame, *args, analyzer=None, **kwargs):
-
+        """ Analysis per frame """
         analysis = {'id': frame.id}
         if analyzer is not None:
             for name,func in analyzer.items():
@@ -205,28 +243,42 @@ class Simulation(list):
 
 
     def make_analysis(self,*args, analyzer=None, **kwargs):
+        """Analyzes the whole simulation (simulation, frames, and objects) """
+        if analyzer is None:
+            analyzer = {}
+
+        for key, lyzer in kwargs.items():
+            if key.endswith("_analyzer"):
+                name = key.split("_analyzer")[0]
+                if name.endswith("s"):
+                    name = name[:-1]
+
+                if name in analyzer.keys():
+                    analyzer[name].update(lyzer)
+                else:
+                    analyzer[name] = lyzer
+
         for frame in self:
             frame.make_analysis(*args, analyzer=analyzer, **kwargs)
 
         self.frames_analysis = an.objects_analysis(self, *args, analyzer=analyzer, **kwargs)
 
-        try:
-            analyzer = analyzer['simulation']
-        except:
-            analyzer = None
+        if "simulation" in analyzer.keys():
+            lyzer = analyzer['simulation']
 
-        analysis = {}
-        if analyzer is not None:
-            for name, func in analyzer.items():
+            analysis = {}
+
+            for name, func in lyzer.items():
                 analysis[name] = func(self)
 
-        if analysis:
-            keys = analysis.keys()
-            datas = pd.DataFrame(columns=keys)
-            datas.loc[self.id] = [analysis[key] for key in keys]
-            self.analysis = datas
+            if analysis:
+                keys = analysis.keys()
+                datas = pd.DataFrame(columns=keys)
+                datas.loc[self.id] = [analysis[key] for key in keys]
+                self.analysis = datas
 
     def show(self, *args, **kwargs):
+        """ Show the plotted system """
         if not __IPV__:
             raise ValueError("Could not import module iPyVolume")
         else:
@@ -244,12 +296,14 @@ class Simulation(list):
                 self[number].plot(*args, **kwargs)
 
     def plot(self, number, *args, **kwargs):
+        """ Plots a frame number """
         if not __IPV__:
             raise ValueError("Could not import module iPyVolume")
         else:
             self[number].plot(*args, **kwargs)
 
     def plot_all(self, *args, **kwargs):
+        """ Plots all frames """
         if not __IPV__:
             raise ValueError("Could not import module iPyVolume")
         else:
@@ -257,6 +311,7 @@ class Simulation(list):
                 frame.plot(*args, **kwargs)
 
     def show_frame(self, number, *args, **kwargs):
+        """ shows a specific frame """
         for frame in self:
             if frame.id == number:
                 frame.show()
@@ -264,6 +319,7 @@ class Simulation(list):
                 frame.hide()
 
     def frame_player(self, interval=2000, *args, **kwargs):
+        """ Shows the system frame by frame """
         if not __IPV__:
             raise ValueError("Could not import module iPyVolume")
         else:
@@ -279,6 +335,7 @@ class Simulation(list):
             #widgets.HBox([play, slider])
 
     def figure(self):
+        """ Whatever that does """
         if not __IPV__:
             raise ValueError("Could not import module iPyVolume")
         else:
